%% Demo_HMRTC_Measured_Signal.m -- Demonstration of the HMRTC recovery method for real MR spectra signals
%% 
%% Reference:
%% [1] Jiaxi Ying, Hengfa Lu, Qingtao Wei, Jian-Feng Cai, Di Guo, Jihui Wu, Zhong Chen, Xiaobo Qu. Hankel matrix nuclear norm regularized tensor completion for N-dimensional exponential signals, IEEE Transactions on Signal Processing, 65(14): 3702-3717, 2017
%% Available: http://dx.doi.org/10.1109/TSP.2017.2695566   or  http://csrc.xmu.edu.cn/pdf/2017_TSP_TensorHankelNMR.pdf 
%% [2] Xiaobo Qu, Maxim Mayzel, Jian-Feng Cai, Zhong Chen, Vladislav Orekhov. Accelerated NMR spectroscopy with low-rank reconstruction, Angewandte Chemie International Edition, 54(3):852-854, 2015. 
%% Available: http://dx.doi.org/10.1002/anie.201409291
%%  
%% By Jiaxi Ying, Xiaobo Qu
%% Computational Sensing Group at Xiamen University     http://www.quxiaobo.org 
%% Email: quxiaobo <at> xmu.edu.cn
%% 2017-05-08
%%
clear;
close all;
clc;
%% add the path
AddPath();
%% Download the realistic 3D NMR data
% The realitic 3D NMR Spectroscopy Data is shared at https://pan.baidu.com/s/1eSswXpS
% The 3-D HNCO spectrum is tested and its sample is the U-[15N,13C] RNA recognition motifs domain of protein RNA binding motif protein 5 [59], which is a component of the spliceosome A-complex.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% If you want to reconstruct the measured signal, first download the Dataset, and put the file 'f1c-t2-t3-NCE.ft' under the following path:  './Data_3DHNCO_nmrPipe'.
filename='f1c-t2-t3-NCE.ft';        % output from nmrPipe
[fin errmsg]=fopen(filename,'r');
if ~isempty(errmsg)
    disp(errmsg);
    disp('Please first download the Dataset, and put the file f1c-t2-t3-NCE.ft in the folder -> Data');
    return;
end
fileheader=fread(fin,512,'float32');
specshape=[1024,2,128,128];
sizespec=prod(specshape);
FID=fread(fin,sizespec,'float32');  % A vector data
%% process the vector to a tensor that is availabe for tensor reconstruction methods
data4D=reshape(FID,specshape);
data3D=data4D(:,1,:,:)+1i*data4D(:,2,:,:);
data3D=squeeze(data3D);             % the direct dimension is in fourier domain
for i=1:128
  for j=1:128
    data(:,i,j)=flipud((ifft(data3D(:,i,j))));
  end
end
data=data/4e4;                     % scale the data
fid3D=data(1:512,:,1:64);          
[row,col,ver]=size(fid3D);
%% load a 3D poisson-gap mask and remove the unsampled data
filename='mask_512_128_64_10%.txt';
mask3D=Readmask3D(size(fid3D,1),size(fid3D,2),size(fid3D,3),filename);  % generate a 3D poisson-gap sampling mask
Omega_mis=(mask3D==0); 
signal_mis=fid3D;
signal_mis(Omega_mis)=0;
%% ADMTR reconstruction
lambda=10;                                 % the regularization parameter for ADM-TR
signal_vec=fid3D(:);
Idx=find(mask3D(:)==1);
B=signal_vec(Idx);
maxiter=1000;                              % the maxium iteration number
VERBOSE=1;
nsignal=norm(fid3D(:));
[sig_ADMTR,Y, n] = tensor_solve_exactADM_final([row,col,ver], Idx, B, maxiter, VERBOSE, fid3D, nsignal,lambda);   % sig_CPTC is the reconctructed data by ADMTR
dif_ADMTRs=norm(fid3D(:)-sig_ADMTR(:))/norm(fid3D(:));             % the reconstruction error by ADM-TR

%% WCP reconstruction
paramsWCP.R=200;                                  % the estimated number of exponentials
paramsWCP.tol=1e-6;                               % the tolerance of the relative difference of cost function in two neighbor iterations 
paramsWCP.MaxIter=1000;                           % the maximum number of iterations
paramsWCP.signal_ori=fid3D;                       % in order to calculate the RLNEs in iterations

sig_WCP= solver_WCP(signal_mis,mask3D,paramsWCP);                     % "sig_WCP" is the result using WCP 
dif_WCP=norm(fid3D(:)-sig_WCP(:))/norm(fid3D(:));                     % the reconstruction error by WCP

%% HMRTC reconstruction
paramsHMRTC_parallel.EstR=500;                                   % the estimated number of exponentials
paramsHMRTC_parallel.lambda=1e3;                                 % the regularization parameter for HMRTC
paramsHMRTC_parallel.rho=1.05;                                   % the parameter for updating beta by beta_k+1=rho*beta_k
paramsHMRTC_parallel.beta=0.1;                                   % the parameter for beta
paramsHMRTC_parallel.tol=1e-4;                                   % the tolerance of the relative difference of outputs of two neighbor iterations 
paramsHMRTC_parallel.MaxIter=1000;                               % the maximum number of iterations
paramsHMRTC_parallel.signal_ori=fid3D;                           % in order to calculate the RLNEs in iterations
paramsHMRTC_parallel.parallelCoreNum=8;                          % the number of parallel cores


sig_HMRTC=solver_HMRTC_ADMM_parallel(signal_mis,mask3D,paramsHMRTC_parallel);  % "sig_HMRTC" is the result using HMRTC 
dif_HMRTC=norm(fid3D(:)-sig_HMRTC(:))/norm(fid3D(:));     

%% plot the reconstructed spectra
% The reconstructed result need some operations to show the spectra well, so we plot the spectra in 'sparky' software instead of MATLAB. 
% If you want to draw the reconstruction spectra, please download 'Data_3DHNCO_nmrPipe' at https://pan.baidu.com/s/1eSswXpS and carefully read Demo_convert_reconstruction_into_nmrPipe.m