%% Demo_HMRTC_Simulated_Signal.m -- Demonstration of the HMRTC recovery method for damped/undamped simulated signals
%% 
%% Reference:
%% [1] Jiaxi Ying, Hengfa Lu, Qingtao Wei, Jian-Feng Cai, Di Guo, Jihui Wu, Zhong Chen, Xiaobo Qu. Hankel matrix nuclear norm regularized tensor completion for N-dimensional exponential signals, IEEE Transactions on Signal Processing, 65(14): 3702-3717, 2017
%% Available: http://dx.doi.org/10.1109/TSP.2017.2695566   or   http://csrc.xmu.edu.cn/pdf/2017_TSP_TensorHankelNMR.pdf
%% [2] Xiaobo Qu, Maxim Mayzel, Jian-Feng Cai, Zhong Chen, Vladislav Orekhov. Accelerated NMR spectroscopy with low-rank reconstruction, Angewandte Chemie International Edition, 54(3):852-854, 2015. 
%% Available: http://dx.doi.org/10.1002/anie.201409291
%%
%% By Jiaxi Ying, Xiaobo Qu
%% Computational Sensing Group at Xiamen University   http://www.quxiaobo.org 
%% Email: quxiaobo <at> xmu.edu.cn
%% 2017-05-08
%%
clear;
close all;
clc;

%% add the path
AddPath();

%%
row=50;                                                    % the ambient dimensions of the signal 50*50*50
col=50;          
ver=50;          
samp_rate=0.5;                                             % Sampling rate using the uniformly random fashion
NumPeaks=50;                                               % the number of exponentials R

%% set rand seed
ctime = datestr(now,30);
tseed = str2num(ctime((end-5):end));
rand('seed',tseed);

%% generate a 3D simulated exponential signal (undamped or damped)

% generate undamped signal
% signal = generate3Dsignal(NumPeaks,row,col,ver); % If you want to obtain reconstructions of undamped signal, please uncomment this line

% generate damped signal
signal = generate_3Ddamping_signal(NumPeaks,row,col,ver); % generate a 3D simulated damped exponential signal with size "row*col*ver" 

spectra3D=fft3c(signal);   
%% add noise
noiseSTD=0.005;                                            % noise level                       
noise=noiseSTD*(randn(size(signal))+1i*randn(size(signal)));
signal_noise=signal+noise;

%% generate a 3D mask and remove the missing data
mask3D = generate_rand_mask(samp_rate,row,col,ver);       % generate a 3D sampling mask
Omega_mis=(mask3D==0); 
signal_mis=signal_noise;
signal_mis(Omega_mis)=0;
spectra3D_mis=fft3c( signal_mis); 
%% ADMTR reconstruction
signal_vec=signal_noise(:);
Idx=find(mask3D(:)==1);
B=signal_vec(Idx);
maxiter=1000;                                                % the maxium iteration number
VERBOSE=0;
nsignal=norm(signal(:));
ADMTR_lambda=10;                                             % the regularization parameter for ADM-TR
t1=clock;
[sig_ADMTR,Y, n] = tensor_solve_exactADM_final([row,col,ver], Idx, B, maxiter, VERBOSE, signal, nsignal,ADMTR_lambda);
t2=clock;
time_ADMTR=etime(t2,t1);
dif_ADMTR=norm(signal(:)-sig_ADMTR(:))/norm(signal(:));      % the reconstruction error by ADM-TR
spectra3D_ADMTR=fft3c(sig_ADMTR); 

%% HMRTC reconstruction
% ============================= Non-parallel version of HMRTC (solver_HMRTC_ADMM) ===========================================
paramsHMRTC.EstR=100;                                   % the estimated number of exponentials
paramsHMRTC.lambda=1e3;                                 % the regularization parameter for HMRTC
paramsHMRTC.rho=1.05;                                   % the parameter for updating beta by beta_k+1=rho*beta_k
paramsHMRTC.beta=0.1;                                   % the parameter for beta
paramsHMRTC.tol=1e-4;                                   % the tolerance of the relative difference of outputs of two neighbor iterations 
paramsHMRTC.MaxIter=1000;                               % the maximum number of iterations
paramsHMRTC.signal_ori=signal;                          % in order to calculate the RLNEs in iterations

t1=clock;
sig_HMRTC=solver_HMRTC_ADMM(signal_mis,mask3D,paramsHMRTC);  % "sig_HMRTC" is the result using HMRTC 
t2=clock;

% ====================== Parallel version of HMRTC HMRTC_parallel (solver_HMRTC_ADMM_parallel)===============================
% paramsHMRTC_parallel.EstR=100;                                   % the estimated number of exponentials
% paramsHMRTC_parallel.lambda=1e3;                                 % the regularization parameter for HMRTC
% paramsHMRTC_parallel.rho=1.05;                                   % the parameter for updating beta by beta_k+1=rho*beta_k
% paramsHMRTC_parallel.beta=0.1;                                   % the parameter for beta
% paramsHMRTC_parallel.tol=1e-4;                                   % the tolerance of the relative difference of outputs of two neighbor iterations 
% paramsHMRTC_parallel.MaxIter=1000;                               % the maximum number of iterations
% paramsHMRTC_parallel.signal_ori=signal;                          % in order to calculate the RLNEs in iterations
% paramsHMRTC_parallel.parallelCoreNum=8;                          % the number of parallel cores

% t1=clock;
% sig_HMRTC=solver_HMRTC_ADMM_parallel(signal_mis,mask3D,paramsHMRTC_parallel);  % "sig_HMRTC" is the result using HMRTC 
% t2=clock;

% ============================================================================================================================

time_HMRTC=etime(t2,t1);
dif_HMRTC=norm(signal(:)-sig_HMRTC(:))/norm(signal(:));                     % the reconstruction error by HMRTC
spectra3D_HMRTC=fft3c(sig_HMRTC); 

%% WCP reconstruction

paramsWCP.R=NumPeaks*2;                           % the estimated number of exponentials
paramsWCP.tol=1e-6;                               % the tolerance of the relative difference of cost function in two neighbor iterations 
paramsWCP.MaxIter=1000;                           % the maximum number of iterations
paramsWCP.signal_ori=signal;                      % in order to calculate the RLNEs in iterations

t1=clock;
sig_WCP= solver_WCP(signal_mis,mask3D,paramsWCP);    % "sig_WCP" is the result using WCP 
t2=clock;

timeWCP=etime(t2,t1);
dif_WCP=norm(signal(:)-sig_WCP(:))/norm(signal(:));           % the reconstruction error by WCP
spectra3D_WCP=fft3c(sig_WCP);

%% draw the spectrum
figure,voxel3((abs(spectra3D)),'fast',false);title('The original spectra')
figure,voxel3(abs(spectra3D_mis),'fast',false);title('The missing spectra')
figure,voxel3((spectra3D_ADMTR),'fast',false);title('The reconstructed spectra by ADMTR')
figure,voxel3((spectra3D_HMRTC),'fast',false);title('The reconstructed spectra by HMRTC')
figure,voxel3((spectra3D_WCP),'fast',false);title('The reconstructed spectra by WCP')
colorbar('FontSize',14);
