% copyright by Silvia Gandy, Tokyo Institute of Technology
% June, 2011

function [Xk,i] = tensor_solve_DR_final(n, Idx, B, maxiter, VERBOSE, Xsol, nXX)
%this function solves a tensor completion problem via the Douglas Rachford
%splitting algorithm
%*********INPUT:************************************ 
%n = [n1, n2, ..., nN] :    sizes of the dimensions of the tensor
%Idx:       vector containing the locations of the known entries
%B:         vector containing the values at the known locations
%maxiter:   maximum number of iterations of the algorithm
%VERBOSE:   controlls the amount of output printed on the screen
%Xsol:      solution tensor (this tensor fulfills Xsol(Idx) == B, however,
%           if for example the number of known locations is low, there might 
%           exist other solutions that have a lower value of the cost function
%nXX:       norm(Xsol)
%*********OUTPUT:************************************ 
%Xk:        solution tensor returned by the Douglas-Rachford based
%           algorithm
%i:         number of iterations after which Xk was obtained (i <= maxiter)

fprintf('\n--------------Algorithm DR-TR------------------------\n\n');

orderX = size(n,2);

dim2_unfold = zeros(1, orderX); % array storing the size of the second dimension of a mode-n unfolding
for i=1:orderX
    dim2_unfold(i) = prod(n([1:(i-1), (i+1):orderX]));
end

%% problem setup -- parameters

%VERBOSE = 0;    % compute the relative error only in the end
%VERBOSE = 1;    % compute and display the relative error in each iteration

VERBOSE_SHOW_RELATIVE_ERROR = VERBOSE; %prints the relative error obtained in each iteration
VERBOSE_SHOW_RELATIVE_ERROR_SAMPLES = 0; %prints the relative error on the sample set in each iteration
VERBOSE_SHOW_DECREASE_COST_FUNCTION = VERBOSE; %prints the value of the cost function ff+gg every 20 iterations

INCREASE_LAMBDA =1;      %shall lambda be increased?

rho_lambda = 2;        %multiplicative factor for lambda
%rho_lambda = 1.2;        %multiplicative factor for lambda
lambda = orderX;
lambda_max = 1e9;        %stop increasing when this value is reached

%% initialize algorithm 

Xk = zeros(n); % n = [n1, n2, n3]
%Xkm1 = Xk;
    
Y = zeros([orderX, n]); %variables from variable splitting, put together in a tensor

r = zeros(1,orderX);

incre = 5;  %increment for the expected rank of Y1, Y2 and Y3

ADJUST_PARAMETERS = 1;%do not adjust parameters in the first iteration

LAMBDA_MAX_REACHED = 0;
COUNT_ADJUST_PARAM = 0; %number of changes of lambda/beta in sequence
MAX_COUNT_ADJUST_PARAM = 8;


Z = zeros(n);
tk_DR = 1;
%what does a good tau look like? link it to beta?
%tauDR = 1*(1+orderX);
tauDR = 1;
TOLERANCE_REL_DELTA_X = 5e-6;



%% run algorithm

if ~VERBOSE
    fprintf('Iteration:     ');
end
for i = 1:maxiter
    if VERBOSE
        fprintf('\nIteration:   %i,  ', i);
    else
        fprintf('\b\b\b\b%4i', i); 
    end
    
    %% adjust parameters
    
    %decide if parameters shall be adjusted in this iteration
    %don't adjust parameters twice in a row
    
    if ~ADJUST_PARAMETERS  % did not adjust parameters in the last iteration
        if (rel_deltaX > 1.2 || rel_deltaX < TOLERANCE_REL_DELTA_X )    
            ADJUST_PARAMETERS =1;            
        else
            COUNT_ADJUST_PARAM = 0;
        end
    else
        ADJUST_PARAMETERS =0; %adjusted params in the last step, keep for this iteration
        COUNT_ADJUST_PARAM = COUNT_ADJUST_PARAM +1; 
    end
    
    %adjust parameters
    if ADJUST_PARAMETERS 
        if INCREASE_LAMBDA 
            if lambda < lambda_max
                lambda =rho_lambda*lambda;
                if VERBOSE
                    fprintf('\blambda adjusted, lambda = %g\n',lambda);
                end
            else
                LAMBDA_MAX_REACHED = 1;
            end
        end
        if ((~INCREASE_LAMBDA)||LAMBDA_MAX_REACHED) 
            fprintf('\nmaximal parameter values reached, stopping\n');
            break;
        end
    end
    
    if COUNT_ADJUST_PARAM == MAX_COUNT_ADJUST_PARAM
        fprintf('\nadjusted parameters %i times in a row, stopping', MAX_COUNT_ADJUST_PARAM);
        break;
    end
  
    %prox for X0 = Z:
    X_hat = 2*Xk - Z;
    X_hat(Idx) = (X_hat(Idx) + lambda*tauDR*B)/(1+lambda*tauDR);
    
    Z = Z + tk_DR*(X_hat - Xk);
    %prox for X1..XN = Y; 
    for iY = 1:orderX
    %% shrinkage Yi (mode-iY unfolding)
    % expected rank of Yi (input to lansvd)
    %reminder: dim2_unfold= prod(n([1:(iY-1), (iY+1):orderX]));
    bound_s = min(n(iY), dim2_unfold(iY));
    s =min(r(iY)+1, bound_s);   

    Yi_unfold = unfold_mode_n_Nthorder(2*Xk- reshape(Y(iY,:),n), iY, n) ;
    
    OK = 0;
    while ~OK  %adjust the input of lansvd
        [U,Sigma,V] = lansvd(Yi_unfold, s,'L');
        OK = (Sigma(s,s) <= tauDR) || ( s == bound_s ) ;
        s = min(s + incre, bound_s);
    end
    sigma = diag(Sigma); r(iY) = sum(sigma > tauDR);
  
    U = U(:,1:r(iY)); V = V(:,1:r(iY)); sigma = sigma(1:r(iY)) - tauDR; Sigma = diag(sigma);
    Yi_shrink = U*diag(sigma)*V';  %% ޸

    Y_tmp = refold_mode_n_Nthorder(Yi_shrink, iY, n);
    Y(iY,:) = Y(iY,:)+ tk_DR*(Y_tmp(:).' - Xk(:).');
    end
    
    %projection onto diagonal subspace
    Xkm1 = Xk;
    Xk = (Z + reshape(sum(Y,1),n))/(orderX+1);
    rel_deltaX = norm(Xk(:)-Xkm1(:))/norm(Xk(:));
    if (~ADJUST_PARAMETERS && VERBOSE)
        fprintf('\b\b\b');
    end
    %fprintf('\brel. delta X = %g', rel_deltaX); 
    

if VERBOSE
    if ~ADJUST_PARAMETERS
        fprintf(', ');
    else
        fprintf('                  ');
    end
    if VERBOSE_SHOW_RELATIVE_ERROR
        res = norm(Xsol(:) - Xk(:));
        rel_err_k = norm(res)/nXX;
        fprintf('relErr: %4g, ', rel_err_k);
    end
    if VERBOSE_SHOW_RELATIVE_ERROR_SAMPLES
        relErrSamples = norm(B - Xk(Idx))/norm(B);
        fprintf('relError/samples: %g, ', relErrSamples);
    end
    if VERBOSE_SHOW_DECREASE_COST_FUNCTION && ~mod(i,20)
        fval = ff_plus_gg(Xk, n, orderX, lambda, B, Idx );
        fprintf('ff + gg; %g, ', fval);
    end
end



end;

end