% copyright by Silvia Gandy, Tokyo Institute of Technology
% June, 2011

function [Xk,Y, i] = tensor_solve_exactADM_final(n, Idx, B, maxiter, VERBOSE, Xsol, nXX,lambda)
%this function solves a tensor completion problem via the ADM algorithm
%(exact version)
%*********INPUT:************************************ 
%n = [n1, n2, ..., nN] :    sizes of the dimensions of the tensor
%Idx:       vector containing the locations of the known entries
%B:         vector containing the values at the known locations
%maxiter:   maximum number of iterations of the algorithm
%VERBOSE:   controlls the amount of output printed on the screen
%Xsol:      solution tensor (this tensor fulfills Xsol(Idx) == B, however,
%           if for example the number of known locations is low, there might 
%           exist other solutions that have a lower value of the cost function
%nXX:       norm(Xsol)
%*********OUTPUT:************************************ 
%Xk:        solution tensor returned by the Douglas-Rachford based
%           algorithm
%Y:         set of tensors, representing the extra Y variables used in the 
%           algorithm
%i:         number of iterations after which Xk was obtained (i <= maxiter)

fprintf('\n--------------Algorithm DR-ADM------------------------\n\n');

orderX = size(n,2);

dim2_unfold = zeros(1, orderX); % array storing the size of the second dimension of a mode-n unfolding
for i=1:orderX
    dim2_unfold(i) = prod(n([1:(i-1), (i+1):orderX]));
end

%% problem setup -- parameters

%VERBOSE = 0;    % compute the relative error only in the end
%VERBOSE = 1;    % compute and display the relative error in each iteration

VERBOSE_SHOW_RELATIVE_ERROR = VERBOSE; %prints the relative error obtained in each iteration
VERBOSE_SHOW_RELATIVE_ERROR_SAMPLES = 0; %prints the relative error on the sample set in each iteration
VERBOSE_SHOW_DECREASE_COST_FUNCTION = VERBOSE; %prints the value of the cost function ff+gg every 20 iterations

INCREASE_BETA =0;        %shall beta be increased?
INCREASE_LAMBDA =0;      %shall lambda be increased?

beta = ones(1,orderX);   %start value of beta (array)

beta_max = 1e5;          %stop increasing when this value is reached
% rho_beta = 1.2;          %multiplicative factor for beta
% rho_lambda = 1.2;        %multiplicative factor for lambda
rho_beta =2;
rho_lambda =2;

% lambda = orderX*beta(1); %start value for lambda

lambda_max = 1e9;        %stop increasing when this value is reached

%% initialize algorithm 

Xk = zeros(n); % n = [n1, n2, n3]
%Xkm1 = Xk;
 
Y = zeros([orderX, n]); %variables from variable splitting, put together in a tensor

W = zeros([orderX, n]); %corresponding Lagrange multiplier

r = zeros(1,orderX);

incre = 5;  %increment for the expected rank of Y1, Y2 and Y3

ADJUST_PARAMETERS = 1;%do not adjust parameters in the first iteration

LAMBDA_MAX_REACHED = 0;
BETA_MAX_REACHED = 0;
COUNT_ADJUST_PARAM = 0; %number of changes of lambda/beta in sequence
MAX_COUNT_ADJUST_PARAM = 8;

%when shall the parameters be adjusted?
TOLERANCE_REL_DELTA_X = 1e-6;


%% run algorithm

if ~VERBOSE
    fprintf('Iteration:     ');
end
for i = 1:maxiter
    if VERBOSE
        fprintf('\nIteration:   %i, ', i);
    else
        fprintf('\b\b\b\b%4i', i); 
    end
    
    %% adjust parameters
    
    %decide if parameters shall be adjusted in this iteration
    %don't adjust parameters twice in a row
    
    if ~ADJUST_PARAMETERS  % did not adjust parameters in the last iteration
        if (rel_deltaX > 1.2 || rel_deltaX < TOLERANCE_REL_DELTA_X )    
            ADJUST_PARAMETERS =1;            
        else
            COUNT_ADJUST_PARAM = 0;
        end
    else
        ADJUST_PARAMETERS =0; %adjusted params in the last step, keep for this iteration
        COUNT_ADJUST_PARAM = COUNT_ADJUST_PARAM +1;  
    end
    
    %adjust parameters
    if ADJUST_PARAMETERS 
        if INCREASE_BETA 
            if beta(1) < beta_max
                  beta = rho_beta*beta; %all beta values are increased identically
                  if VERBOSE 
                      % only printing beta(1)
                      fprintf('beta adjusted, beta(1) =%g, lambda =%g,\n', beta(1), lambda);
                  end
            else 
                BETA_MAX_REACHED = 1;
            end
        end;
        if INCREASE_LAMBDA 
            if lambda < lambda_max
                lambda =rho_lambda*lambda;
                if VERBOSE
                    fprintf('lambda adjusted, beta(1) =%g, lambda = %g\n',beta(1), lambda);
                end
            else
                LAMBDA_MAX_REACHED = 1;
            end
        end
        if ((~INCREASE_LAMBDA)||LAMBDA_MAX_REACHED) && ((~INCREASE_BETA)||BETA_MAX_REACHED)
            fprintf('\nmaximal parameter values reached, stopping\n');
            break;
        end
    end
    
    if COUNT_ADJUST_PARAM == MAX_COUNT_ADJUST_PARAM
        fprintf('\nadjusted parameters %i times in a row, stopping', MAX_COUNT_ADJUST_PARAM);
        break;
    end




%% update step X

%evaluate A^*(b-A(X))

Y_tmp = zeros(n);
for iY = 1:orderX 
    Y_tmp = Y_tmp + reshape(beta(iY)*Y(iY, :), n);
end

Xkm1 = Xk;       
sumW_plus_sumBetaY = reshape(sum(W,1), n) + Y_tmp;
sum_beta = sum(beta);
Xk = sumW_plus_sumBetaY/sum_beta;
Xk(Idx) = (sumW_plus_sumBetaY(Idx) + lambda*B)/(lambda+sum_beta);
rel_deltaX = norm(Xk(:)-Xkm1(:))/norm(Xk(:));
if (~ADJUST_PARAMETERS && VERBOSE)
    fprintf('\b\b');
end
  %fprintf('\brelative change of X = %g', rel_deltaX);
       
for iY =1:orderX

% expected rank of Yi (input to lansvd)
%dim2_unfold= prod(n([1:(iY-1), (iY+1):orderX]));
bound_s = min(n(iY), dim2_unfold(iY));
s =min(r(iY)+1, bound_s);   

%% shrinkage Yi (mode-iY unfolding)

Yi_unfold = unfold_mode_n_Nthorder(Xk- reshape(1/beta(iY)*W(iY,:),n), iY, n) ;
tau = 1/beta(iY);  % input to the shrinkage operator
OK = 0;
while ~OK  %adjust the input of lansvd
%% Modification 
% The original codes shared by authors use function "lansvd" to compute SVD
% [U,Sigma,V] = lansvd(Yi_unfold, s,'L');
% However, the function "lansvd" cannot support complex number, hence we do some
% modification by using "svd", the MATLAB function, instead of "lansvd"
% Jiaxi Ying, May 8, 2017
%%
   [U,S,V]=svd(Yi_unfold);
   U=U(:,1:s);
   V=V(:,1:s);
   Sigma=S(1:s,1:s);
   OK = (Sigma(s,s) <= tau) || ( s == bound_s ) ;
   s = min(s + incre, bound_s);
end
sigma = diag(Sigma); r(iY) = sum(sigma > tau);
  
U = U(:,1:r(iY)); V = V(:,1:r(iY)); sigma = sigma(1:r(iY)) - tau; Sigma = diag(sigma);
Yi_shrink = U*diag(sigma)*V';

Y_tmp = refold_mode_n_Nthorder(Yi_shrink, iY, n);  
Y(iY,:) = Y_tmp(:);
end

%% update laplace multiplier
for iW = 1:orderX
    W(iW,:) = W(iW,:) - beta(iW)*(Xk(:).' - Y(iW,:)); 
end

if VERBOSE
    if ~ADJUST_PARAMETERS
        fprintf(', ');  
    else
        fprintf('                  ');
    end
    if VERBOSE_SHOW_RELATIVE_ERROR
        res = norm(Xsol(:) - Xk(:));
        rel_err_k = norm(res)/nXX;
        fprintf('relErr: %4g, ', rel_err_k);
    end
    if VERBOSE_SHOW_RELATIVE_ERROR_SAMPLES
        relErrSamples = norm(B - Xk(Idx))/norm(B);
        fprintf('relError/samples: %g, ', relErrSamples);
    end
    if VERBOSE_SHOW_DECREASE_COST_FUNCTION && ~mod(i,20)
        fval = ff_plus_gg(Xk, n, orderX, lambda, B, Idx );
        fprintf('ff + gg; %g, ', fval);
    end
    
end



end;

end